@@ -1,7 +1,7 @@
 # coding=utf8
 """
 Label propagation in the context of this module refers to a set of
-semisupervised classification algorithms. In the high level, these algorithms
+semi-supervised classification algorithms. At a high level, these algorithms
 work by forming a fully-connected graph between all points given and solving
 for the steady-state distribution of labels at each point.
 
@@ -14,27 +14,29 @@ For more information see the references below.
 Model Features
 --------------
 Label clamping:
-  The algorithm tries to learn distributions of labels over the dataset. In the
-  "Hard Clamp" mode, the true ground labels are never allowed to change. They
-  are clamped into position. In the "Soft Clamp" mode, they are allowed some
-  wiggle room, but some alpha of their original value will always be retained.
-  Hard clamp is the same as soft clamping with alpha set to 1.
+  The algorithm tries to learn distributions of labels over the dataset given
+  label assignments over an initial subset. In one variant, the algorithm does
+  not allow for any errors in the initial assignment (hard-clamping) while
+  in another variant, the algorithm allows for some wiggle room for the initial
+  assignments, allowing them to change by a fraction alpha in each iteration
+  (soft-clamping).
 
 Kernel:
   A function which projects a vector into some higher dimensional space. This
-  implementation supprots RBF and KNN kernels. Using the RBF kernel generates
+  implementation supports RBF and KNN kernels. Using the RBF kernel generates
   a dense matrix of size O(N^2). KNN kernel will generate a sparse matrix of
   size O(k*N) which will run much faster. See the documentation for SVMs for
   more info on kernels.
 
 Examples
 --------
+>>> import numpy as np
 >>> from sklearn import datasets
 >>> from sklearn.semi_supervised import LabelPropagation
 >>> label_prop_model = LabelPropagation()
 >>> iris = datasets.load_iris()
->>> random_unlabeled_points = np.where(np.random.random_integers(0, 1,
-...        size=len(iris.target)))
+>>> rng = np.random.RandomState(42)
+>>> random_unlabeled_points = rng.rand(len(iris.target)) < 0.3
 >>> labels = np.copy(iris.target)
 >>> labels[random_unlabeled_points] = -1
 >>> label_prop_model.fit(iris.data, labels)
@@ -52,26 +54,23 @@ Non-Parametric Function Induction in Semi-Supervised Learning. AISTAT 2005
 """
 
 # Authors: Clay Woolam <clay@woolam.org>
-# Licence: BSD
+#          Utkarsh Upadhyay <mail@musicallyut.in>
+# License: BSD
 from abc import ABCMeta, abstractmethod
-from scipy import sparse
+
+import warnings
 import numpy as np
+from scipy import sparse
+from scipy.sparse import csgraph
 
 from ..base import BaseEstimator, ClassifierMixin
-from ..metrics.pairwise import rbf_kernel
-from ..utils.graph import graph_laplacian
-from ..utils.extmath import safe_sparse_dot
-from ..utils.validation import check_X_y, check_is_fitted, check_array
-from ..utils.multiclass import check_classification_targets
 from ..externals import six
+from ..metrics.pairwise import rbf_kernel
 from ..neighbors.unsupervised import NearestNeighbors
-
-
-### Helper functions
-
-def _not_converged(y_truth, y_prediction, tol=1e-3):
-    """basic convergence check"""
-    return np.abs(y_truth - y_prediction).sum() > tol
+from ..utils.extmath import safe_sparse_dot
+from ..utils.multiclass import check_classification_targets
+from ..utils.validation import check_X_y, check_is_fitted, check_array
+from ..exceptions import ConvergenceWarning
 
 
 class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
@@ -80,30 +79,37 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
 
     Parameters
     ----------
-    kernel : {'knn', 'rbf'}
-        String identifier for kernel function to use.
-        Only 'rbf' and 'knn' kernels are currently supported..
+    kernel : {'knn', 'rbf', callable}
+        String identifier for kernel function to use or the kernel function
+        itself. Only 'rbf' and 'knn' strings are valid inputs. The function
+        passed should take two inputs, each of shape [n_samples, n_features],
+        and return a [n_samples, n_samples] shaped weight matrix
 
     gamma : float
         Parameter for rbf kernel
 
+    n_neighbors : integer > 0
+        Parameter for knn kernel
+
     alpha : float
         Clamping factor
 
-    max_iter : float
+    max_iter : integer
         Change maximum number of iterations allowed
 
     tol : float
         Convergence tolerance: threshold to consider the system at steady
         state
 
-    n_neighbors : integer > 0
-        Parameter for knn kernel
-
+   n_jobs : int or None, optional (default=None)
+        The number of parallel jobs to run.
+        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        for more details.
     """
 
     def __init__(self, kernel='rbf', gamma=20, n_neighbors=7,
-                 alpha=1, max_iter=30, tol=1e-3):
+                 alpha=1, max_iter=30, tol=1e-3, n_jobs=None):
 
         self.max_iter = max_iter
         self.tol = tol
@@ -116,6 +122,8 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
         # clamping factor
         self.alpha = alpha
 
+        self.n_jobs = n_jobs
+
     def _get_kernel(self, X, y=None):
         if self.kernel == "rbf":
             if y is None:
@@ -124,16 +132,23 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
                 return rbf_kernel(X, y, gamma=self.gamma)
         elif self.kernel == "knn":
             if self.nn_fit is None:
-                self.nn_fit = NearestNeighbors(self.n_neighbors).fit(X)
+                self.nn_fit = NearestNeighbors(self.n_neighbors,
+                                               n_jobs=self.n_jobs).fit(X)
             if y is None:
                 return self.nn_fit.kneighbors_graph(self.nn_fit._fit_X,
                                                     self.n_neighbors,
                                                     mode='connectivity')
             else:
                 return self.nn_fit.kneighbors(y, return_distance=False)
+        elif callable(self.kernel):
+            if y is None:
+                return self.kernel(X, X)
+            else:
+                return self.kernel(X, y)
         else:
             raise ValueError("%s is not a valid kernel. Only rbf and knn"
-                             " are supported at this time" % self.kernel)
+                             " or an explicit function "
+                             " are supported at this time." % self.kernel)
 
     @abstractmethod
     def _build_graph(self):
@@ -174,8 +189,8 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
         """
         check_is_fitted(self, 'X_')
 
-        X_2d = check_array(X, accept_sparse = ['csc', 'csr', 'coo', 'dok',
-                        'bsr', 'lil', 'dia'])
+        X_2d = check_array(X, accept_sparse=['csc', 'csr', 'coo', 'dok',
+                                             'bsr', 'lil', 'dia'])
         weight_matrices = self._get_kernel(self.X_, X_2d)
         if self.kernel == 'knn':
             probabilities = []
@@ -225,10 +240,13 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
 
         n_samples, n_classes = len(y), len(classes)
 
+        alpha = self.alpha
+        if self._variant == 'spreading' and \
+                (alpha is None or alpha <= 0.0 or alpha >= 1.0):
+            raise ValueError('alpha=%s is invalid: it must be inside '
+                             'the open interval (0, 1)' % alpha)
         y = np.asarray(y)
         unlabeled = y == -1
-        clamp_weights = np.ones((n_samples, 1))
-        clamp_weights[unlabeled, 0] = self.alpha
 
         # initialize distributions
         self.label_distributions_ = np.zeros((n_samples, n_classes))
@@ -236,32 +254,52 @@ class BaseLabelPropagation(six.with_metaclass(ABCMeta, BaseEstimator,
             self.label_distributions_[y == label, classes == label] = 1
 
         y_static = np.copy(self.label_distributions_)
-        if self.alpha > 0.:
-            y_static *= 1 - self.alpha
-        y_static[unlabeled] = 0
+        if self._variant == 'propagation':
+            # LabelPropagation
+            y_static[unlabeled] = 0
+        else:
+            # LabelSpreading
+            y_static *= 1 - alpha
 
         l_previous = np.zeros((self.X_.shape[0], n_classes))
 
-        remaining_iter = self.max_iter
+        unlabeled = unlabeled[:, np.newaxis]
         if sparse.isspmatrix(graph_matrix):
             graph_matrix = graph_matrix.tocsr()
-        while (_not_converged(self.label_distributions_, l_previous, self.tol)
-                and remaining_iter > 1):
+
+        for self.n_iter_ in range(self.max_iter):
+            if np.abs(self.label_distributions_ - l_previous).sum() < self.tol:
+                break
+
             l_previous = self.label_distributions_
             self.label_distributions_ = safe_sparse_dot(
                 graph_matrix, self.label_distributions_)
-            # clamp
-            self.label_distributions_ = np.multiply(
-                clamp_weights, self.label_distributions_) + y_static
-            remaining_iter -= 1
+
+            if self._variant == 'propagation':
+                normalizer = np.sum(
+                    self.label_distributions_, axis=1)[:, np.newaxis]
+                self.label_distributions_ /= normalizer
+                self.label_distributions_ = np.where(unlabeled,
+                                                     self.label_distributions_,
+                                                     y_static)
+            else:
+                # clamp
+                self.label_distributions_ = np.multiply(
+                    alpha, self.label_distributions_) + y_static
+        else:
+            warnings.warn(
+                'max_iter=%d was reached without convergence.' % self.max_iter,
+                category=ConvergenceWarning
+            )
+            self.n_iter_ += 1
 
         normalizer = np.sum(self.label_distributions_, axis=1)[:, np.newaxis]
         self.label_distributions_ /= normalizer
+
         # set the transduction item
         transduction = self.classes_[np.argmax(self.label_distributions_,
                                                axis=1)]
         self.transduction_ = transduction.ravel()
-        self.n_iter_ = self.max_iter - remaining_iter
         return self
 
 
@@ -272,9 +310,11 @@ class LabelPropagation(BaseLabelPropagation):
 
     Parameters
     ----------
-    kernel : {'knn', 'rbf'}
-        String identifier for kernel function to use.
-        Only 'rbf' and 'knn' kernels are currently supported..
+    kernel : {'knn', 'rbf', callable}
+        String identifier for kernel function to use or the kernel function
+        itself. Only 'rbf' and 'knn' strings are valid inputs. The function
+        passed should take two inputs, each of shape [n_samples, n_features],
+        and return a [n_samples, n_samples] shaped weight matrix.
 
     gamma : float
         Parameter for rbf kernel
@@ -283,15 +323,25 @@ class LabelPropagation(BaseLabelPropagation):
         Parameter for knn kernel
 
     alpha : float
-        Clamping factor
+        Clamping factor.
 
-    max_iter : float
+        .. deprecated:: 0.19
+            This parameter will be removed in 0.21.
+            'alpha' is fixed to zero in 'LabelPropagation'.
+
+    max_iter : integer
         Change maximum number of iterations allowed
 
     tol : float
         Convergence tolerance: threshold to consider the system at steady
         state
 
+    n_jobs : int or None, optional (default=None)
+        The number of parallel jobs to run.
+        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        for more details.
+
     Attributes
     ----------
     X_ : array, shape = [n_samples, n_features]
@@ -311,12 +361,13 @@ class LabelPropagation(BaseLabelPropagation):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from sklearn import datasets
     >>> from sklearn.semi_supervised import LabelPropagation
     >>> label_prop_model = LabelPropagation()
     >>> iris = datasets.load_iris()
-    >>> random_unlabeled_points = np.where(np.random.random_integers(0, 1,
-    ...    size=len(iris.target)))
+    >>> rng = np.random.RandomState(42)
+    >>> random_unlabeled_points = rng.rand(len(iris.target)) < 0.3
     >>> labels = np.copy(iris.target)
     >>> labels[random_unlabeled_points] = -1
     >>> label_prop_model.fit(iris.data, labels)
@@ -333,6 +384,15 @@ class LabelPropagation(BaseLabelPropagation):
     --------
     LabelSpreading : Alternate label propagation strategy more robust to noise
     """
+
+    _variant = 'propagation'
+
+    def __init__(self, kernel='rbf', gamma=20, n_neighbors=7,
+                 alpha=None, max_iter=1000, tol=1e-3, n_jobs=None):
+        super(LabelPropagation, self).__init__(
+            kernel=kernel, gamma=gamma, n_neighbors=n_neighbors, alpha=alpha,
+            max_iter=max_iter, tol=tol, n_jobs=n_jobs)
+
     def _build_graph(self):
         """Matrix representing a fully connected graph between each sample
 
@@ -349,11 +409,20 @@ class LabelPropagation(BaseLabelPropagation):
             affinity_matrix /= normalizer[:, np.newaxis]
         return affinity_matrix
 
+    def fit(self, X, y):
+        if self.alpha is not None:
+            warnings.warn(
+                "alpha is deprecated since 0.19 and will be removed in 0.21.",
+                DeprecationWarning
+            )
+            self.alpha = None
+        return super(LabelPropagation, self).fit(X, y)
+
 
 class LabelSpreading(BaseLabelPropagation):
     """LabelSpreading model for semi-supervised learning
 
-    This model is similar to the basic Label Propgation algorithm,
+    This model is similar to the basic Label Propagation algorithm,
     but uses affinity matrix based on the normalized graph Laplacian
     and soft clamping across the labels.
 
@@ -361,9 +430,11 @@ class LabelSpreading(BaseLabelPropagation):
 
     Parameters
     ----------
-    kernel : {'knn', 'rbf'}
-        String identifier for kernel function to use.
-        Only 'rbf' and 'knn' kernels are currently supported.
+    kernel : {'knn', 'rbf', callable}
+        String identifier for kernel function to use or the kernel function
+        itself. Only 'rbf' and 'knn' strings are valid inputs. The function
+        passed should take two inputs, each of shape [n_samples, n_features],
+        and return a [n_samples, n_samples] shaped weight matrix
 
     gamma : float
       parameter for rbf kernel
@@ -372,15 +443,25 @@ class LabelSpreading(BaseLabelPropagation):
       parameter for knn kernel
 
     alpha : float
-      clamping factor
+      Clamping factor. A value in [0, 1] that specifies the relative amount
+      that an instance should adopt the information from its neighbors as
+      opposed to its initial label.
+      alpha=0 means keeping the initial label information; alpha=1 means
+      replacing all initial information.
 
-    max_iter : float
+    max_iter : integer
       maximum number of iterations allowed
 
     tol : float
       Convergence tolerance: threshold to consider the system at steady
       state
 
+    n_jobs : int or None, optional (default=None)
+        The number of parallel jobs to run.
+        ``None`` means 1 unless in a :obj:`joblib.parallel_backend` context.
+        ``-1`` means using all processors. See :term:`Glossary <n_jobs>`
+        for more details.
+
     Attributes
     ----------
     X_ : array, shape = [n_samples, n_features]
@@ -400,12 +481,13 @@ class LabelSpreading(BaseLabelPropagation):
 
     Examples
     --------
+    >>> import numpy as np
     >>> from sklearn import datasets
     >>> from sklearn.semi_supervised import LabelSpreading
     >>> label_prop_model = LabelSpreading()
     >>> iris = datasets.load_iris()
-    >>> random_unlabeled_points = np.where(np.random.random_integers(0, 1,
-    ...    size=len(iris.target)))
+    >>> rng = np.random.RandomState(42)
+    >>> random_unlabeled_points = rng.rand(len(iris.target)) < 0.3
     >>> labels = np.copy(iris.target)
     >>> labels[random_unlabeled_points] = -1
     >>> label_prop_model.fit(iris.data, labels)
@@ -423,14 +505,17 @@ class LabelSpreading(BaseLabelPropagation):
     LabelPropagation : Unregularized graph based semi-supervised learning
     """
 
+    _variant = 'spreading'
+
     def __init__(self, kernel='rbf', gamma=20, n_neighbors=7, alpha=0.2,
-                 max_iter=30, tol=1e-3):
+                 max_iter=30, tol=1e-3, n_jobs=None):
 
         # this one has different base parameters
         super(LabelSpreading, self).__init__(kernel=kernel, gamma=gamma,
                                              n_neighbors=n_neighbors,
                                              alpha=alpha, max_iter=max_iter,
-                                             tol=tol)
+                                             tol=tol,
+                                             n_jobs=n_jobs)
 
     def _build_graph(self):
         """Graph matrix for Label Spreading computes the graph laplacian"""
@@ -439,7 +524,7 @@ class LabelSpreading(BaseLabelPropagation):
             self.nn_fit = None
         n_samples = self.X_.shape[0]
         affinity_matrix = self._get_kernel(self.X_)
-        laplacian = graph_laplacian(affinity_matrix, normed=True)
+        laplacian = csgraph.laplacian(affinity_matrix, normed=True)
         laplacian = -laplacian
         if sparse.isspmatrix(laplacian):
             diag_mask = (laplacian.row == laplacian.col)
